/* general */
body {
    padding-bottom:44px;
    overflow-y:scroll;
}

/* main nav bar */
.navbar-top {
    border-radius:0;
    border-width:0 0 1px;
}

/* search field */
#search {
    padding-left:10px;
    padding-top:10px;
    padding-right:10px;
}

/* navtree */
#navtree li {
    padding-left:8px;
}
#navtree li > a {
    padding:6px 10px 6px 0px;
}
#navtree li > a:hover {
    color:#333;
    background-color:transparent;
}
#navtree li.active > a {
    font-weight:bold;
}
#navtree li.navtreelevel_1 {
    padding-left:0px;
}
#navtree li.navtreelevel_2 {
    margin-left:6px;
    border-left:#cdcdcd 3px solid;
}
#navtree li.navtreelevel_2 > a {
    padding:5px 10px 5px 0px;
}
#navtree li.navtreelevel_3 {
    margin-left:5px;
    border-left:#dddddd 3px solid;
}
#navtree li.navtreelevel_3 > a {
    padding:4px 10px 4px 0px;
}
#navtree li.navtreelevel_4 {
    margin-left:4px;
    border-left:#dedede 3px solid;
}
#navtree li.navtreelevel_4 > a {
    padding:3px 10px 3px 0px;
}
#navtree li.navtreelevel_5 {
    margin-left:4px;
    border-left:#eeeeee 3px solid;
}
#navtree li.navtreelevel_5 > a {
    padding:2px 10px 2px 0px;
}
#navtree li.navtreelevel_6 {
    margin-left:4px;
    border-left:#efefef 3px solid;
}
#navtree li.navtreelevel_6 > a {
    padding:2px 10px 2px 0px;
}

/* content panel heading */
.content-heading h4 {
    padding-left: 0;
}

/* batched items */
.batched_items_slice_size {
    margin-top: 2px;
}
.batched_items_filter {
    margin-right:8px;
    margin-top:2px;
}
.batched_items_footer .pagination {
    margin-top:0;
    margin-bottom:0;
}
.batched_items_info {
    margin-top:4px;
    margin-bottom:4px;
}
input.empty_filter {
    color:#ccc !important;
}

/* table */
.table_length {
    margin-top:2px;
}
.table_filter {
    margin-right:8px;
    margin-top:2px;
}
.table_sort {
    font-size:80%;
    color:#428BCA;
}
table.table tr th:first-child,
table.table tr td:first-child {
    padding-left:20px;
}
td.actions a {
    margin-right:4px;
}
.table-footer .pagination {
    margin-top:0;
    margin-bottom:0;
}
.table_info {
    margin-top:4px;
    margin-bottom:4px;
}

/* copysupport */
tr.selectable td {
    user-select: none;
    -moz-user-select: none;
    -webkit-user-select: none;
    -ms-user-select: none;
}
tr.selectable.selected td {
    background-color:#dfeefb;
}
.table-striped > tbody > tr.selectable.selected:nth-child(2n+1) > td {
    background-color:#cfdce7;
}

/* referencebrowser */
.referencebrowser_trigger {
    padding:10px 15px 10px 5px;
    cursor:pointer;
}
#referencebrowser_table td.actions {
    width:100px;
}
.modal-body div.referencebrowser {
    margin:-15px;
}
div#referencebrowser_pathbar ol.breadcrumb {
    border-radius:0px;
}
div#referencebrowser {
    margin-bottom:0px;
    border-radius:0px;
    border-left:none;
    border-right:none;
    border-bottom:none;
}
div#referencebrowser .panel-heading,
div#referencebrowser .panel-footer {
    border-radius:0px;
}

/* settings */
.settingstabpane {
    margin-top:12px;
}

/* footer */
footer p {
    margin:12px;
}